"""
Calculating number of occurrences of each code, Cohen's kappa and Brennan Prediger's kappa

The raw data that is needed to run this:
- post_questionnaire_coder1.csv
- post_questionnaire_coder2.xlsx

The processed data that is generated is printed in the terminal.

Author: Yoan Naydenov
Date: June 2024
"""

import pandas as pd
from sklearn.metrics import cohen_kappa_score


def count_code_occurrences(file_path_coded):
    # Load the dataset
    data = pd.read_csv(file_path_coded)


    data.columns = ['rand_id', 'code1', 'code2']

    # Combine the codes from both columns into a single Series
    codes_combined = pd.concat([data['code1'], data['code2']])

    # Count the occurrences of each code
    code_frequencies = codes_combined.value_counts()

    # Return the result
    return code_frequencies


def calculate_category_kappas(file1: str, file2: str) -> dict:
    categories = [
        "no difference", "real person", "helpful", "support", "personalised", "emotional and understanding",
        "repeats kai", "different perspective/expert", "engagement and interaction", "accountability",
        "encouraging", "more motivating", "neutral", "basic response", "trust", "good/helpful advice",
        "provoke thinking", "kai is better", "reminding", "effectiveness and reliability", "guidance",
        "valuable", "rude", "validating", "wrong suggestion"
    ]

    # Read the CSV and Excel files
    df1 = pd.read_csv(file1)
    df2 = pd.read_excel(file2)

    # Ensure all codes are lowercase for case-insensitive matching
    df1['code1'] = df1['code1'].str.lower()
    df1['code2'] = df1['code2'].str.lower()
    df2['code1'] = df2['code1'].str.lower()
    df2['code2'] = df2['code2'].str.lower()

    # Extract relevant columns for the first and second codes from both dataframes
    df1_code1 = df1[['rand_id', 'code1', 'code2']].rename(columns={'code1': 'code1_coder1', 'code2': 'code2_coder1'})
    df2_code1 = df2[['rand_id', 'code1', 'code2']].rename(columns={'code1': 'code1_coder2', 'code2': 'code2_coder2'})

    # Merge the dataframes on 'rand_id'
    merged_df = pd.merge(df1_code1, df2_code1, on='rand_id')

    # Initialize a dictionary to store kappa values for each category
    kappa_values = {}

    # Calculate Cohen's kappa for each category
    for category in categories:
        # Create binary presence/absence indicators for each category
        merged_df[f'{category}_coder1'] = merged_df.apply(lambda row: category in str(row['code1_coder1']) or category
                                                                      in str(row['code2_coder1']), axis=1)
        merged_df[f'{category}_coder2'] = merged_df.apply(lambda row: category in str(row['code1_coder2']) or category
                                                                      in str(row['code2_coder2']), axis=1)

        # Calculate Cohen's kappa for the category
        if merged_df[f'{category}_coder1'].nunique() > 1 and merged_df[f'{category}_coder2'].nunique() > 1:
            kappa = cohen_kappa_score(merged_df[f'{category}_coder1'], merged_df[f'{category}_coder2'])
        else:
            kappa = float('nan')  # Not enough data to calculate kappa

        kappa_values[category] = kappa

    return kappa_values


def calculate_weighted_average_kappa(category_kappas: dict, occurrences: pd.Series) -> float:

    # Ensure that the keys in category_kappas and occurrences match
    valid_categories = [category for category in category_kappas if category in occurrences.index]

    total_weighted_kappa = sum(category_kappas[category] * int(occurrences[category]) for category in valid_categories)

    total_occurrences = sum(int(occurrences[category]) for category in valid_categories)

    weighted_average_kappa = total_weighted_kappa / total_occurrences
    return weighted_average_kappa


def calculate_brennan_prediger_kappa(file1: str, file2: str) -> tuple:
    # Read the CSV and Excel files
    df1 = pd.read_csv(file1)
    df2 = pd.read_excel(file2)

    # Extract relevant columns for the first and second codes from both dataframes
    df1_code1 = df1[['rand_id', 'code1']].rename(columns={'code1': 'code1_coder1'})
    df2_code1 = df2[['rand_id', 'code1']].rename(columns={'code1': 'code1_coder2'})

    df1_code2 = df1[['rand_id', 'code2']].rename(columns={'code2': 'code2_coder1'})
    df2_code2 = df2[['rand_id', 'code2']].rename(columns={'code2': 'code2_coder2'})

    merged_code1 = pd.merge(df1_code1, df2_code1, on='rand_id')
    merged_code2 = pd.merge(df1_code2, df2_code2, on='rand_id')

    # Combine the merged dataframes
    combined_df = pd.concat([merged_code1, merged_code2], ignore_index=True)

    # Drop rows where either coder has NaN values
    combined_df.dropna(subset=['code1_coder1', 'code1_coder2'], inplace=True)

    # Debug: Check if any data is left after dropping NaNs
    if combined_df.empty:
        print("No valid data to process after dropping NaNs.")
        return 0, 0, 0

    # Calculate observed agreement
    combined_agreement = combined_df['code1_coder1'] == combined_df['code1_coder2']
    P_o_combined = combined_agreement.mean()

    # Number of unique categories
    combined_categories = pd.concat([combined_df['code1_coder1'], combined_df['code1_coder2']]).unique()
    k_combined = len(combined_categories)


    # Check for zero unique categories to avoid division by zero
    if k_combined == 0:
        print("Zero unique categories found.")
        return 0, 0, 0

    # Calculate expected agreement
    P_e_combined = 1 / k_combined

    # Calculate Brennan-Prediger kappa
    kappa_BP_combined = (P_o_combined - P_e_combined) / (1 - P_e_combined)

    return P_o_combined, P_e_combined, kappa_BP_combined


first_coder_path = ''
second_coder_path = ''


if __name__ == '__main__':
    kappa_per_code = calculate_category_kappas(first_coder_path, second_coder_path)
    print(kappa_per_code)

    code_occurrences = count_code_occurrences(first_coder_path)
    print(code_occurrences)

    weighted_kappa = calculate_weighted_average_kappa(kappa_per_code, code_occurrences)
    print(weighted_kappa)

    brennan_prediger_kappa = calculate_brennan_prediger_kappa(first_coder_path, second_coder_path)
    print(brennan_prediger_kappa)
