"""
Calculating correlation between each theme and a subset of user characteristics.

The raw data that is needed to run this:
- a csv file with all codes in it and the variables as well.

The processed data that is generated is printed in the terminal.

Author: Yoan Naydenov
Date: June 2024
"""

import pandas as pd
from scipy.stats import spearmanr


def analyze_theme_correlations(data_file_path, quantitative_columns):
    # Load the quantitative data and codes file
    data = pd.read_csv(data_file_path)

    # Mapping codes to themes
    code_to_theme = {
        'trust': 1,
        'validating': 1,
        'emotional and understanding': 1,
        'support': 1,
        'real person': 1,
        'guidance': 2,
        'valuable': 2,
        'personalised': 2,
        'different perspective/expert': 2,
        'good/helpful advice': 2,
        'reminding': 3,
        'helpful': 3,
        'provoke thinking': 3,
        'engagement and interaction': 3,
        'effectiveness and reliability': 3,
        'more motivating': 4,
        'encouraging': 4,
        'accountability': 5
    }

    # Function to count the number of codes related to each theme in each response
    def count_codes_themes(row):
        codes = [row['code1'], row['code2']]
        theme_counts = {1: 0, 2: 0, 3: 0, 4: 0, 5: 0}
        for code in codes:
            if code in code_to_theme:
                theme_counts[code_to_theme[code]] = 1
        return theme_counts

    # Apply the function to create new columns for each theme count
    theme_counts_df = data.apply(count_codes_themes, axis=1, result_type='expand')
    theme_counts_df.columns = ['theme1', 'theme2', 'theme3', 'theme4', 'theme5']

    # Merge the theme counts with the original data
    data_with_counts = pd.concat([data, theme_counts_df], axis=1)

    # Select columns for correlation analysis

    correlation_data_counts = data_with_counts[['theme1', 'theme2', 'theme3', 'theme4',
                                                'theme5'] + quantitative_columns].dropna()

    # Calculate Spearman correlation coefficients and p-values
    def spearman_corr_pvals(df, col1, col2):
        corr, pval = spearmanr(df[col1], df[col2])
        return corr, pval

    # Calculate correlation coefficients and p-values for each pair of theme counts and quantitative data
    results = {}
    for theme in ['theme1', 'theme2', 'theme3', 'theme4', 'theme5']:
        for col in quantitative_columns:
            corr, pval = spearman_corr_pvals(correlation_data_counts, theme, col)
            results[(theme, col)] = (corr, pval)

    # Convert results to a DataFrame for easier viewing
    results_df = pd.DataFrame(results, index=['Spearman Correlation', 'p-value']).T

    return results_df


quantitative_columns1 = ['age', 'P_engagement_R_1', 'P_accountability_R_1', 'P_effect_R_1', 'P_with_without_R_1']

quantitative_columns2 = ['QSI_Smoke_1', 'QSI_Smoke_2', 'QSI_Smoke_3', 'Smoking_Freq', 'Weekly_Exercise']

file_path = ''
if __name__ == '__main__':
    correlation = analyze_theme_correlations(file_path,
                                             quantitative_columns1)
    print(correlation)
    correlation = analyze_theme_correlations(file_path,
                                             quantitative_columns2)
    print(correlation)
